@@ -11,29 +11,25 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 import logging
-import functools
 
 from botocore import waiter, xform_name
+from botocore.args import ClientArgsCreator
 from botocore.auth import AUTH_TYPE_MAPS
 from botocore.awsrequest import prepare_request_dict
 from botocore.docs.docstring import ClientMethodDocstring
 from botocore.docs.docstring import PaginatorDocstring
-from botocore.exceptions import ClientError, DataNotFoundError
-from botocore.exceptions import OperationNotPageableError
-from botocore.exceptions import UnknownSignatureVersionError
+from botocore.exceptions import (
+    DataNotFoundError, OperationNotPageableError, UnknownSignatureVersionError,
+    InvalidEndpointDiscoveryConfigurationError, UnknownFIPSEndpointError,
+)
 from botocore.hooks import first_non_none_response
 from botocore.model import ServiceModel
 from botocore.paginate import Paginator
-from botocore.utils import CachedProperty
-from botocore.utils import get_service_module_name
-from botocore.utils import S3RegionRedirector
-from botocore.utils import S3ArnParamHandler
-from botocore.utils import S3EndpointSetter
-from botocore.args import ClientArgsCreator
-from botocore import UNSIGNED
-# Keep this imported.  There's pre-existing code that uses
-# "from botocore.client import Config".
-from botocore.config import Config
+from botocore.utils import (
+    CachedProperty, get_service_module_name, S3RegionRedirector,
+    S3ArnParamHandler, S3EndpointSetter, ensure_boolean,
+    S3ControlArnParamHandler, S3ControlEndpointSetter,
+)
 from botocore.history import get_global_history_recorder
 from botocore.discovery import (
     EndpointDiscoveryHandler, EndpointDiscoveryManager,
@@ -42,6 +38,15 @@ from botocore.discovery import (
 from botocore.retries import standard
 from botocore.retries import adaptive
 
+# Keep these imported.  There's pre-existing code that uses:
+# "from botocore.client import Config"
+# "from botocore.client import ClientError"
+# etc.
+from botocore.config import Config # noqa
+from botocore.exceptions import ClientError # noqa
+from botocore.args import ClientArgsCreator # noqa
+from botocore import UNSIGNED # noqa
+
 
 logger = logging.getLogger(__name__)
 history_recorder = get_global_history_recorder()
@@ -88,9 +93,13 @@ class ClientCreator(object):
         self._register_s3_events(
             service_client, endpoint_bridge, endpoint_url, client_config,
             scoped_config)
+        self._register_s3_control_events(
+            service_client, endpoint_bridge, endpoint_url, client_config,
+            scoped_config)
         self._register_endpoint_discovery(
             service_client, endpoint_url, client_config
         )
+        self._register_lazy_block_unknown_fips_pseudo_regions(service_client)
         return service_client
 
     def create_client_class(self, service_name, api_version=None):
@@ -197,14 +206,62 @@ class ClientCreator(object):
         elif self._config_store:
             enabled = self._config_store.get_config_variable(
                 'endpoint_discovery_enabled')
-        if enabled:
-            manager = EndpointDiscoveryManager(client)
+
+        enabled = self._normalize_endpoint_discovery_config(enabled)
+        if enabled and self._requires_endpoint_discovery(client, enabled):
+            discover = enabled is True
+            manager = EndpointDiscoveryManager(client, always_discover=discover)
             handler = EndpointDiscoveryHandler(manager)
             handler.register(events, service_id)
         else:
             events.register('before-parameter-build',
                             block_endpoint_discovery_required_operations)
 
+    def _normalize_endpoint_discovery_config(self, enabled):
+        """Config must either be a boolean-string or string-literal 'auto'"""
+        if isinstance(enabled, str):
+            enabled = enabled.lower().strip()
+            if enabled == 'auto':
+                return enabled
+            elif enabled in ('true', 'false'):
+                return ensure_boolean(enabled)
+        elif isinstance(enabled, bool):
+            return enabled
+
+        raise InvalidEndpointDiscoveryConfigurationError(config_value=enabled)
+
+    def _requires_endpoint_discovery(self, client, enabled):
+        if enabled == "auto":
+            return client.meta.service_model.endpoint_discovery_required
+        return enabled
+
+    def _register_lazy_block_unknown_fips_pseudo_regions(self, client):
+        # This function blocks usage of FIPS pseudo-regions when the endpoint
+        # is not explicitly known to exist to the client to prevent accidental
+        # usage of incorrect or non-FIPS endpoints. This is done lazily by
+        # registering an exception on the before-sign event to allow for
+        # general client usage such as can_paginate, exceptions, etc.
+        region_name = client.meta.region_name
+        if not region_name or 'fips' not in region_name.lower():
+            return
+
+        partition = client.meta.partition
+        endpoint_prefix = client.meta.service_model.endpoint_prefix
+        known_regions = self._endpoint_resolver.get_available_endpoints(
+            endpoint_prefix,
+            partition,
+            allow_non_regional=True,
+        )
+
+        if region_name not in known_regions:
+            def _lazy_fips_exception(**kwargs):
+                service_name = client.meta.service_model.service_name
+                raise UnknownFIPSEndpointError(
+                    region_name=region_name,
+                    service_name=service_name,
+                )
+            client.meta.events.register('before-sign', _lazy_fips_exception)
+
     def _register_s3_events(self, client, endpoint_bridge, endpoint_url,
                             client_config, scoped_config):
         if client.meta.service_model.service_name != 's3':
@@ -221,6 +278,21 @@ class ClientCreator(object):
         self._set_s3_presign_signature_version(
             client.meta, client_config, scoped_config)
 
+    def _register_s3_control_events(
+        self, client, endpoint_bridge,
+        endpoint_url, client_config, scoped_config
+    ):
+        if client.meta.service_model.service_name != 's3control':
+            return
+        S3ControlArnParamHandler().register(client.meta.events)
+        S3ControlEndpointSetter(
+            endpoint_resolver=self._endpoint_resolver,
+            region=client.meta.region_name,
+            s3_config=client.meta.config.s3,
+            endpoint_url=endpoint_url,
+            partition=client.meta.partition
+        ).register(client.meta.events)
+
     def _set_s3_presign_signature_version(self, client_meta,
                                           client_config, scoped_config):
         # This will return the manually configured signature version, or None
@@ -395,8 +467,9 @@ class ClientEndpointBridge(object):
             else:
                 # Use the sslCommonName over the hostname for Python 2.6 compat.
                 hostname = resolved.get('sslCommonName', resolved.get('hostname'))
-                endpoint_url = self._make_url(hostname, is_secure,
-                                            resolved.get('protocols', []))
+                endpoint_url = self._make_url(
+                    hostname, is_secure, resolved.get('protocols', [])
+                )
         signature_version = self._resolve_signature_version(
             service_name, resolved)
         signing_name = self._resolve_signing_name(service_name, resolved)
